@@ -94,8 +94,8 @@ namespace xt
         {
             using value_type = T;
 
-            arange_impl(T start, T stop, T step) :
-                m_start(start), m_stop(stop), m_step(step)
+            arange_impl(T start, T stop, T step)
+                : m_start(start), m_stop(stop), m_step(step)
             {
             }
 
@@ -188,13 +188,7 @@ namespace xt
             template <class It>
             inline T operator()(const It& /*begin*/, const It& end) const
             {
-                // workaround windows compile error by using temporary 
-                // iterators and operator-=
-                auto end_1 = end;
-                auto end_2 = end;
-                end_1 -= 1;
-                end_2 -= 2;
-                return *(end_1) == *(end_2) + m_k ? T(1) : T(0);
+                return *(end - 1) == *(end - 2) + m_k ? T(1) : T(0);
             }
 
         private:
@@ -289,40 +283,41 @@ namespace xt
     template <class T>
     inline auto logspace(T start, T stop, std::size_t num_samples, T base = 10, bool endpoint = true) noexcept
     {
-        return pow(base, linspace(start, stop, num_samples, endpoint));
+        return pow(std::forward<T>(base), linspace(start, stop, num_samples, endpoint));
     }
 
     namespace detail
     {
-        template <class... T>
+        template <class... CT>
         struct concatenate_impl
         {
             using size_type = std::size_t;
-            using value_type = std::common_type_t<typename std::decay_t<T>::value_type...>;
+            using value_type = std::common_type_t<typename std::decay_t<CT>::value_type...>;
 
-            concatenate_impl(std::tuple<T...>&& t, std::size_t axis) :
-                m_t(t), m_axis(axis)
+            inline concatenate_impl(std::tuple<CT...>&& t, std::size_t axis)
+                : m_t(t), m_axis(axis)
             {
             }
 
             template <class... Args>
-            value_type operator()(Args... args) const
+            inline value_type operator()(Args... args) const
             {
                 return access_impl(xindex({{static_cast<size_type>(args)...}}));
             }
 
-            value_type operator[](const xindex& idx) const
+            inline value_type operator[](const xindex& idx) const
             {
                 return access_impl(idx);
             }
 
             template <class It>
-            value_type element(It first, It last) const
+            inline value_type element(It first, It last) const
             {
                 return access_impl(xindex(first, last));
             }
 
         private:
+
             inline value_type access_impl(xindex idx) const
             {
                 auto match = [this, &idx](auto& arr) {
@@ -339,7 +334,7 @@ namespace xt
                 };
 
                 std::size_t i = 0;
-                for (; i < sizeof...(T); ++i)
+                for (; i < sizeof...(CT); ++i)
                 {
                     if (apply<bool>(i, match, m_t)) 
                     {
@@ -349,70 +344,108 @@ namespace xt
                 return apply<value_type>(i, get, m_t);
             }
 
-            std::tuple<T...> m_t;
+            std::tuple<CT...> m_t;
             size_type m_axis;
         };
 
-        template <class... T>
+        template <class... CT>
         struct stack_impl
         {
             using size_type = std::size_t;
-            using value_type = std::common_type_t<typename std::decay_t<T>::value_type...>;
+            using value_type = std::common_type_t<typename std::decay_t<CT>::value_type...>;
 
-            stack_impl(std::tuple<T...>&& t, std::size_t axis) :
-                m_t(t), m_axis(axis)
+            inline stack_impl(std::tuple<CT...>&& t, std::size_t axis)
+                : m_t(t), m_axis(axis)
             {
             }
 
             template <class... Args>
-            value_type operator()(Args... args) const
+            inline value_type operator()(Args... args) const
             {
                 return access_impl(xindex({{static_cast<size_type>(args)...}}));
             }
 
-            value_type operator[](const xindex& idx) const
+            inline value_type operator[](const xindex& idx) const
             {
                 return access_impl(idx);
             }
 
             template <class It>
-            value_type element(It first, It last) const
+            inline value_type element(It first, It last) const
             {
                 return access_impl(xindex(first, last));
             }
 
         private:
+
             inline value_type access_impl(xindex idx) const
             {
                 auto get_item = [&idx](auto& arr) {
                     return arr[idx];
                 };
                 std::size_t i = idx[m_axis];
-                // TODO workaround MSVC iterator bug
-                auto it = idx.begin();
-                it += m_axis;
-                idx.erase(it);
+                idx.erase(idx.begin() + m_axis);
                 return apply<value_type>(i, get_item, m_t);
             }
-            const std::tuple<T...> m_t;
+
+            const std::tuple<CT...> m_t;
             const size_type m_axis;
         };
+
+        template <class CT>
+        struct repeat_impl
+        {
+            using xexpression_type = std::decay_t<CT>;
+            using size_type = typename xexpression_type::size_type;
+            using value_type = typename xexpression_type::value_type;
+
+            repeat_impl(CT source, size_type axis) :
+                m_source(source), m_axis(axis)
+            {
+            }
+
+            template <class... Args>
+            value_type operator()(Args... args) const
+            {
+                // to catch a -Wuninitialized
+                if (sizeof...(Args))
+                {
+                    std::array<size_type, sizeof...(Args)> args_arr({static_cast<size_type>(args)...});
+                    return m_source(args_arr[m_axis]);
+                }
+                return m_source();
+            }
+
+            value_type operator[](const xindex& idx) const
+            {
+                return m_source[{ idx[m_axis] }];
+            }
+
+            template <class It>
+            inline value_type element(It first, It) const
+            {
+                return m_source[{*(first + m_axis)}];
+            }
+
+        private:
+
+            CT m_source;
+            size_type m_axis;
+        };
     }
 
     /**
-     * @function xtuple
      * @brief Creates tuples from arguments for \ref concatenate and \ref stack.
      *        Very similar to std::make_tuple.
      */
     template <class... Types>
-    auto xtuple(Types&&... args)
+    inline auto xtuple(Types&&... args)
     {
         return std::tuple<detail::const_closure_t<Types>...>(std::forward<Types>(args)...);
     }
 
     /**
-     * @function concatentate
-     * @brief Concatenate xexpressions along \em axis.
+     * @brief Concatenates xexpressions along \em axis.
      *
      * @param t \ref xtuple of xexpressions to concatenate
      * @param axis axis along which elements are concatenated
@@ -426,16 +459,16 @@ namespace xt
      * xt::xarray<double> d = xt::concatenate(xt::xtuple(a, b), 1); // => {{1, 2, 3, 2, 3, 4}}
      * \endcode
      */
-    template <class... T>
-    inline auto concatenate(std::tuple<T...>&& t, std::size_t axis = 0)
+    template <class... CT>
+    inline auto concatenate(std::tuple<CT...>&& t, std::size_t axis = 0)
     {
-        using shape_type = promote_shape_t<typename std::decay_t<T>::shape_type...>;
+        using shape_type = promote_shape_t<typename std::decay_t<CT>::shape_type...>;
         shape_type new_shape = forward_sequence<shape_type>(std::get<0>(t).shape());
         auto shape_at_axis = [&axis](std::size_t prev, auto& arr) -> std::size_t {
             return prev + arr.shape()[axis];
         };
         new_shape[axis] += accumulate(shape_at_axis, std::size_t(0), t) - new_shape[axis];
-        return detail::make_xgenerator(detail::concatenate_impl<T...>(std::forward<std::tuple<T...>>(t), axis), new_shape);
+        return detail::make_xgenerator(detail::concatenate_impl<CT...>(std::forward<std::tuple<CT...>>(t), axis), new_shape);
     }
 
     namespace detail
@@ -444,21 +477,9 @@ namespace xt
         inline std::array<T, N + 1> add_axis(std::array<T, N> arr, std::size_t axis, std::size_t value)
         {
             std::array<T, N + 1> temp;
-            for (std::size_t i = 0; i < N + 1; ++i)
-            {
-                if (i < axis)
-                {
-                    temp[i] = arr[i];
-                }
-                if (i == axis)
-                {
-                    temp[i] = value;
-                }
-                if (i > axis)
-                {
-                    temp[i] = arr[i - 1];
-                }
-            }
+            std::copy(arr.begin(), arr.begin() + axis, temp.begin());
+            temp[axis] = value;
+            std::copy(arr.begin() + axis, arr.end(), temp.begin() + axis + 1);
             return temp;
         }
 
@@ -466,16 +487,12 @@ namespace xt
         inline T add_axis(T arr, std::size_t axis, std::size_t value)
         {
             T temp(arr);
-            // TODO workaround MSVC iterator bug
-            auto it = temp.begin();
-            it += axis;
-            temp.insert(it, value);
+            temp.insert(temp.begin() + axis, value);
             return temp;
         }
     }
 
     /**
-     * @function stack
      * @brief Stack xexpressions along \em axis.
      *        Stacking always creates a new dimension along which elements are stacked.
      *
@@ -493,12 +510,51 @@ namespace xt
      *                                                               {3, 7}}
      * \endcode
      */
-    template <class... T>
-    inline auto stack(std::tuple<T...>&& t, std::size_t axis = 0)
+    template <class... CT>
+    inline auto stack(std::tuple<CT...>&& t, std::size_t axis = 0)
     {
-        using shape_type = promote_shape_t<typename std::decay_t<T>::shape_type...>;
-        auto new_shape = detail::add_axis(forward_sequence<shape_type>(std::get<0>(t).shape()), axis, sizeof...(T));
-        return detail::make_xgenerator(detail::stack_impl<T...>(std::forward<std::tuple<T...>>(t), axis), new_shape);
+        using shape_type = promote_shape_t<typename std::decay_t<CT>::shape_type...>;
+        auto new_shape = detail::add_axis(forward_sequence<shape_type>(std::get<0>(t).shape()), axis, sizeof...(CT));
+        return detail::make_xgenerator(detail::stack_impl<CT...>(std::forward<std::tuple<CT...>>(t), axis), new_shape);
+    }
+
+    namespace detail
+    {
+
+        template <std::size_t... I, class... E>
+        inline auto meshgrid_impl(std::index_sequence<I...>, E&&... e) noexcept
+        {
+#if defined X_OLD_CLANG || defined _MSC_VER
+            const std::array<std::size_t, sizeof...(E)> shape { e.shape()[0]... };
+            return std::make_tuple(
+                detail::make_xgenerator(
+                    detail::repeat_impl<xclosure_t<E>>(std::forward<E>(e), I),
+                    shape
+                )...
+            );
+#else
+            return std::make_tuple(
+                detail::make_xgenerator(
+                    detail::repeat_impl<xclosure_t<E>>(std::forward<E>(e), I),
+                    { e.shape()[0]... }
+                )...
+            );
+#endif
+        }
+    }
+
+    /**
+     * @brief Return coordinate tensors from coordinate vectors.
+     *        Make N-D coordinate tensor expressions for vectorized evaluations of N-D scalar/vector
+     *        fields over N-D grids, given one-dimensional coordinate arrays x1, x2,..., xn.
+     *
+     * @param e xexpressions to concatenate
+     * @returns tuple of xgenerator expressions.
+     */
+    template <class... E>
+    inline auto meshgrid(E&&... e) noexcept
+    {
+        return detail::meshgrid_impl(std::make_index_sequence<sizeof...(E)>(), std::forward<E>(e)...);
     }
 }
 #endif
