@@ -31,45 +31,51 @@ namespace xt
          ********************/
 
         template <class... Args>
-        struct common_size_type_impl
+        struct common_size_type
         {
             using type = std::common_type_t<typename Args::size_type...>;
         };
 
         template <>
-        struct common_size_type_impl<>
+        struct common_size_type<>
         {
             using type = std::size_t;
         };
 
         template<class... Args>
-        using common_size_type = typename common_size_type_impl<Args...>::type;
+        using common_size_type_t = typename common_size_type<Args...>::type;
 
         /**************************
          * common_difference type *
          **************************/
 
         template <class... Args>
-        struct common_difference_type_impl
+        struct common_difference_type
         {
             using type = std::common_type_t<typename Args::difference_type...>;
         };
 
         template <>
-        struct common_difference_type_impl<>
+        struct common_difference_type<>
         {
             using type = std::size_t;
         };
 
         template<class... Args>
-        using common_difference_type = typename common_difference_type_impl<Args...>::type;
+        using common_difference_type_t = typename common_difference_type<Args...>::type;
 
         /*********************
          * common_value_type *
          *********************/
 
         template <class... Args>
-        using common_value_type = std::common_type_t<xvalue_type_t<Args>...>;
+        struct common_value_type
+        {
+            using type = std::common_type_t<xvalue_type_t<Args>...>;
+        };
+
+        template <class... Args>
+        using common_value_type_t = typename common_value_type<Args...>::type;
     }
 
     template <class F, class R, class... CT>
@@ -107,8 +113,8 @@ namespace xt
         using const_reference = value_type;
         using pointer = value_type*;
         using const_pointer = const value_type*;
-        using size_type = detail::common_size_type<std::decay_t<CT>...>;
-        using difference_type = detail::common_difference_type<std::decay_t<CT>...>;
+        using size_type = detail::common_size_type_t<std::decay_t<CT>...>;
+        using difference_type = detail::common_difference_type_t<std::decay_t<CT>...>;
 
         using shape_type = promote_shape_t<typename std::decay_t<CT>::shape_type...>;
 
@@ -179,6 +185,8 @@ namespace xt
         template <class Func, std::size_t... I>
         const_iterator build_iterator(Func&& f, std::index_sequence<I...>) const noexcept;
 
+        size_type compute_dimension() const noexcept;
+
         std::tuple<CT...> m_e;
         functor_type m_f;
         shape_type m_shape;
@@ -305,7 +313,7 @@ namespace xt
     template <class F, class R, class... CT>
     template <class Func>
     inline xfunction<F, R, CT...>::xfunction(Func&& f, CT... e) noexcept
-        : m_e(e...), m_f(std::forward<Func>(f)), m_shape(make_sequence<shape_type>(dimension(), size_type(1)))
+        : m_e(e...), m_f(std::forward<Func>(f)), m_shape(make_sequence<shape_type>(compute_dimension(), size_type(1)))
     {
         broadcast_shape(m_shape);
     }
@@ -321,8 +329,7 @@ namespace xt
     template <class F, class R, class... CT>
     inline auto xfunction<F, R, CT...>::dimension() const noexcept -> size_type
     {
-        auto func = [](size_type d, auto&& e) noexcept { return std::max(d, e.dimension()); };
-        return accumulate(func, size_type(0), m_e);
+        return m_shape.size();
     }
 
     /**
@@ -495,7 +502,7 @@ namespace xt
     /**
      * Returns a constant iterator to the first element of the function. The
      * iteration is broadcasted to the specified shape.
-     * @param shape the shape used for braodcasting
+     * @param shape the shape used for broadcasting
      */
     template <class F, class R, class... CT>
     template <class S>
@@ -519,7 +526,7 @@ namespace xt
     /**
      * Returns a constant iterator to the first element of the function. The
      * iteration is broadcasted to the specified shape.
-     * @param shape the shape used for braodcasting
+     * @param shape the shape used for broadcasting
      */
     template <class F, class R, class... CT>
     template <class S>
@@ -585,6 +592,13 @@ namespace xt
         return const_iterator(this, f(std::get<I>(m_e))...);
     }
 
+    template <class F, class R, class... CT>
+    inline auto xfunction<F, R, CT...>::compute_dimension() const noexcept -> size_type
+    {
+        auto func = [](size_type d, auto&& e) noexcept { return std::max(d, e.dimension()); };
+        return accumulate(func, size_type(0), m_e);
+    }
+
     /*************************************
      * xfunction_iterator implementation *
      *************************************/
