@@ -371,7 +371,7 @@ namespace xt
     /**
      * Returns a constant iterator to the first element of the function. The
      * iteration is broadcasted to the specified shape.
-     * @param shape the shape used for braodcasting
+     * @param shape the shape used for broadcasting
      */
     template <class F, class R, class S>
     template <class O>
@@ -395,7 +395,7 @@ namespace xt
     /**
      * Returns a constant iterator to the first element of the function. The
      * iteration is broadcasted to the specified shape.
-     * @param shape the shape used for braodcasting
+     * @param shape the shape used for broadcasting
      */
     template <class F, class R, class S>
     template <class O>
@@ -521,10 +521,10 @@ namespace xt
 #endif
         
         template <class Functor, class S>
-        inline auto make_xgenerator(Functor&& f, const S& shape) noexcept
+        inline auto make_xgenerator(Functor&& f, S&& shape) noexcept
         {
-            using type = xgenerator<Functor, typename Functor::value_type, S>;
-            return type(std::forward<Functor>(f), shape);
+            using type = xgenerator<Functor, typename Functor::value_type, std::decay_t<S>>;
+            return type(std::forward<Functor>(f), std::forward<S>(shape));
         }
     }
 }
